function [MV,b,Lambda]=problem0a(n)
%[MV,b]=problem0(n) 
%  The same as 'problem0', except for some commands that allow to inspect
%  the effect of preconditioning on the parsity of the matrices and on
%  the eigenvalues.
%
%[MV,b,Lambda]=problem0(n)
%   Lambda is an n x 1 array of eigenvalues of A
%   (returns Lambda=[] if n is too large to allow efficient computation).
%
%Note: a function-handle is used to avoid repeatedly defining the matrix A.
%   See MATLAB's documentation "Using Handles to Store Function Parameters".
%

  %%% put INSPECT=1 for graphical output.
  INSPECT=1;
  
  %%% Define the matrix H
  %%% Run "help gallery" or for information on matrices in the set 
  %%% "gallery" of test matrices available in Matlab,
  %%% run "help private/poisson" for information on the specific matrix
  %%% called poisson (from a discretization of Poisson's Equation).
  
  H=gallery('poisson',n);
  
  %%% determine the dimension of the problem: H is a sparse N x N matrix.
  
  N=size(H,1);
  
  %%% Determine b.
  %%% In practise b is given. However, for test purposes, it is convenient
  %%% to define b from a convenient solution x (which, of course, generally
  %%% is the quatity that has to be computed)
  
  x=ones(N,1); b=H*x;
  
  %%% Here we change the problem to demonstate the advantage of using 
  %%% functions to perform the matrix-vector multiplication rather
  %%% than using matrices: here, we "precondition" H by it's upper and lower
  %%% triangular parts (including diagonal). This results in the matrix A.
   
  L=tril(H);
  U=triu(H);
  D=diag(diag(H));
  
  
  if  INSPECT %% Check the sparsity of matrices that are involved
    figure(1), spy(H), 
    title(sprintf('H:=Poisson(%d), an %d x %d matrix',n,N,N))
    %% Check the sparsity if L,U,D 
    figure(2), spy(abs(L)+abs(U)+abs(D));
    title('Lower and upper triangular and  diagonal part of H.')
    %% Check the sparsity of A:= D*(L\(H/U)).
    A=D*(L\(H/U)); figure(3), spy(A)
    title ('Sparsity of A, preconditioned H.')
  end 

  %%% Define the matrix-vector multiplication with A:= D*(L\(H/U)).
  function c=MyMV(u)
     c=D*(L\(H*(U\u)));
  end
   
  %%% We are interested in solving D*L\(H/U)y=D*(L\b) for y. Then x=U\y.
  b=D*(L\b);
  
  MV=@MyMV;
 
  
  
  
  %%%  if requested, compute the eigenvalues of H and of A if N is not too large
  if (nargout==3 & n<51) 
     %%% From "help private/poisson":
     %%% The matrix is symmettric,  positive definite and has eigenvalues
     %%%   2*( 2 - cos(pi*i/(n+1)) - cos(pi*j/(n+1)) ), i = 1:n, j = 1:n.
     X=ones(n,1)*(1:n); I=reshape(X,N,1); J=reshape(X',N,1);
     Lambda=2*(2-cos(I*(pi/(n+1)))-cos(J*(pi/(n+1))));
     Lambda=sort(Lambda); 
     if INSPECT 
       %%% plot the eigenvalues of A scaled such that the largest eigenvalue is 1
       Lmax=max(Lambda);
       figure, plot(Lambda/Lmax,0*Lambda,'r*')
     end
     %%% compute the eigenvalues of A
     Lambda=sort(eig(D*(L\(H/U)))); 
     if INSPECT %%% plot the scaled eigenvalues of A to see the effect of
                %%% preconditioning on the eigenvalues
         Lmax=max(Lambda); hold on, plot(Lambda/Lmax,0*Lambda,'g+')
     legend('Lambda(H)','Lambda(A)')
     title('The eigenvalues of H and A, scaled to have largest value 1.')
    end      
  end
   

end