function [x,hist,t,NoMV]=arnoldi(A,v,tol,kmax,flag)

  %%% set defaults
  if nargin<5, flag='lr'; end
  if nargin<4, kmax=100; end
  if nargin<3, tol=1.0e-8; end
  
  %%% Is A a matrix?
  A_IS_MATRIX = IsMatrix(A); NoMV=0;
  
  %%% Define the matrix-vector multiplication MV.
  function c=MV(x)
    if A_IS_MATRIX, c=A*x; else, c=feval(A,x); end
    %%% Count the number of matrix vector multiplications.
    NoMV=NoMV+1;
  end
  
  %%% One Arnoldi step for expanding the Arnoldi decomposition
  %%% Use kappa to select the orthogonalisation procedure:
  %%%     for the options, see 'Orth'
  kappa=-1;
  function [V,H]=ArnStep(V,H)
     w=MV(V(:,k));          %%% Compute w, the expansion vector
     [v,h]=Orth(V,w,kappa); %%% Orthogonalise w against V 
     V=[V,v];               %%% Expand the Arnoldi decomposition
     if size(v,2), H=[H;zeros(1,k-1)]; end
     H=[H,h];
  end

  t0=clock; 
  %%%
  [n,m]=size(v); kmax=min(kmax,n);

  Lambda=PlotEigs(A,n);

  %%% The Arnoldi loop
  rho=norm(v); hist=[];
  V=[v/rho]; H=zeros(1,0); gamma=1; nu=1;
  for k=1:kmax
     if rho<tol, break, end
     [V,H]=ArnStep(V,H);      %%% Expand the Arnoldi decomposition
     [Y,D]=eig(H(1:k,:)); Ritz=diag(D); %%% compute the pre-Ritz pairs
     I=mysort(Ritz,flag); Ritz=Ritz(I); Y=Y(:,I); %%% select pre-Ritz pair
     PlotEigs(A,n,Ritz);      %%% Plot the Ritz values
     y=Y(:,1); y=y/norm(y,2); %%% (Re)scale the selected pre-Ritz vector
     %%% compute residual norm
     if size(H,1)>k, rho=abs(H(k+1,k)*y(k,1)); else, rho=0; end
     hist=[hist,rho];
  end
  
  x=V(:,1:size(y))*y;      %%% Compute the Ritz vector

  t=etime(clock,t0);
  
  %----------------------------------------------------
  %%% Set PLOTEIGS=0, if no graphical output is needed
  PLOTEIGS=1;
  
  function LAMBDA=PlotEigs(A,n,Ritz)
  
     if (n>2000 &(~PLOTEIGS)), return, end

     if nargin==2, 
       %%% compute the matrix if A is a function
       if A_IS_MATRIX, AA=A; else, AA=MV(eye(n)); end
       LAMBDA=eig(AA); 
       return, 
     end

     plot(real(Lambda),imag(Lambda),'*'), hold on
     plot(real(Ritz),imag(Ritz),'ro'), hold off
     title(sprintf('Ritz values of order %d    (%d eigenvalues)',length(Ritz),length(Lambda)))
     legend('\lambda','\theta')
     pause(0.1)

   end


end
%=======================================
function I=mysort(Lambda,flag)
%%% To do: different selection criterio depending on the value of flag

       %%% select the Ritz value with largest real part
       [Lambda,I]=sort(real(Lambda));

end
%======================================
