close all
clear all

if exist('gcr')~=2, install, end


tol=1.0e-12;
kmax=800;

DEFINE_YOUR_OWN_MATRIX=1;

if DEFINE_YOUR_OWN_MATRIX

  %%% dimension of the matrix
  n=1000; 
  MVtype='handle'; MVtype= 'function'; MVtype= 'matrix';
  switch MVtype
    case 'handle'
      %%% A is the handle of the function that performs the MV as
      %%% defined in 'problem1a'
      [A,b,Lambda]=problem1a(n);
    case 'function'  
      %%% A is the name of the function that performs the MV
      A='problem1b'; [b,Lambda]=feval(A,[],n);
    case 'matrix'
      %%% A is the name of the matrix defined in the function 'problem1c'
      [A,b,Lambda]=problem1c(n);
  end
  
  % [A,b,Lambda]=problem3(n);  

else

  NameMatrix='Grond2e3';  % NameMatrix='meier01';
  [A,b,hbtype]=matrixhbo(NameMatrix); b=b(:,1); n=size(b,1);
  fprintf(1,'\n\n%s: %d by %d matrix of type %s\n\n',NameMatrix,n,n,hbtype)
  
end

n=size(b,1);

%%% If desired for testing, redefine the righthand side vector b.
if 0
  x=rand(n,1);
  x=ones(n,1);
  if IsMatrix(A), b=A*x; else, b=feval(A,x); end
  b=b/norm(b);
end

%%% Is an interval Lambda that contains all eigenvalues available?
INTERVAL=exist('Lambda');
if INTERVAL, INTERVAL=(INTERVAL & length(Lambda)>1); end

if INTERVAL
   ell=8;   
   %%% find zeros x of the Chebyshev polynomial of degree ell
   x=cos(((1:ell)-0.5)*pi/ell);
   %%% find centre and radius of the interval that contains the eigenvalues 
   Lmin=min(Lambda); Lmax=max(Lambda);    %%% spectral ends
   mu=0.5*(Lmax+Lmin);                    %%% centre spectrum
   radius=0.5*(Lmax-Lmin);                %%% radius spectrum
   %%% shift x to the zeros to the interval Lambda that contains the eigenvalues 
   Mu=mu-radius*x; 
   %%% sort Mu
   I=myorder(Mu); Mu=Mu(I);
end 


myplot();
Methods=strvcat('Rich','LMR','ChebFix','Cheb','GCR');
Methods=strvcat('ChebFix','Cheb','GCR','GMRES','FOM');

for k=1:size(Methods,1)
method=deblank(Methods(k,:));

if (strcmp(method,'Rich') & INTERVAL)
%%% Richardson iteration with optimal shift
  [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax,mu);
  myplot(A,b,x,hist,t,NoMV,method,'b');
end

if strcmp(method,'LMR')
  %%% LMR
  [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'r');
end

if (strcmp(method,'ChebFix') & INTERVAL)
%%% Chebyshev fixed polynomial with optimal shifts
  [x,hist,t,NoMV]=polynomialsolver(A,b,0*b,tol,kmax,Mu);
  method=sprintf('Cheb(%d)',length(Mu));
  myplot(A,b,x,hist,t,NoMV,method,'m');
end

if (strcmp(method,'Cheb') & INTERVAL)
  %%% Chebyshev (three-term recurrence)
  [x,hist,t,NoMV]=chebyshev(A,b,0*b,mu,radius,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'g');
end

if strcmp(method,'GCR')
  %%% GCR
  L=1;U=1;
  % Aa=MyMATRIX; U=triu(Aa); L=tril(Aa)/diag(diag(Aa));
  [x,hist,t,NoMV]=gcr(A,b,0*b,tol,kmax,L,U);
  myplot(A,b,x,hist,t,NoMV,method,'k');
end

if strcmp(method,'GMRES')
  %%% GMRES
  [x,hist,t,NoMV]=gmres(A,b,0*b,tol,kmax);
  myplot(A,b,x,hist,t,NoMV,method,'b');
end

if strcmp(method,'FOM')
  %%% FOM
  [x,hist,t,NoMV]=gmres(A,b,0*b,tol,kmax,'FOM');
  myplot(A,b,x,hist,t,NoMV,method,'r');
end

end

myplot(A,b)


