function I=IsMatrix(A)
%I=IsMatrix(A)
%  I=1 if A is a matrix
%  I=0 if A is a function handle or the name (i.e., a string) of a function. 
%  An error message is produced in other cases.

  if nargin==0, I=-1; return, end

  I = ~isa(A,'function_handle');
  if (I & ischar(A)), I = (exist(A)~=2); end
  
  if I, 
    if isnumeric(A), 
       I=1;
    else, 
       str1=sprintf('  "%s" is of unknown type:',inputname(1)) 
       str2=sprintf('\n  "%s" should be either a matrix of numerics,',inputname(1))
       str3=sprintf('\n  a function handle, or a name (string) of a function.')
       str4=sprintf('\n\n  In case of a function (identified by a handle or a name),')
       str5=sprintf('\n  the function should perform the matrix-vector multiplication.\n\n')
       error([str1,str2,str3,str4,str5])
    end, 
  end
  
 
return
